import { defineNuxtModule, createResolver, addComponent, addImports, addServerImports, addTemplate, useNitro } from '@nuxt/kit';

const name = "nuxt-shiki";
const version = "0.2.1";

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "shiki"
  },
  defaults: {
    bundledLangs: ["typescript", "javascript", "json"],
    bundledThemes: ["min-light", "min-dark"]
  },
  setup(options, nuxt) {
    const resolver = createResolver(import.meta.url);
    addUnwasmSupport(nuxt);
    addComponent({
      filePath: resolver.resolve("./runtime/Shiki"),
      name: "Shiki"
    });
    addImports([
      {
        name: "loadShiki",
        from: resolver.resolve("./runtime/loadShiki")
      },
      {
        name: "useHighlighted",
        from: resolver.resolve("./runtime/useHighlighted")
      }
    ]);
    addServerImports([
      {
        name: "loadShiki",
        from: resolver.resolve("./runtime/loadShiki")
      }
    ]);
    const bundledThemes = Array.from(
      /* @__PURE__ */ new Set([
        ...options.bundledThemes || [],
        ...typeof options.defaultTheme === "string" ? [options.defaultTheme] : Object.values(options.defaultTheme || {})
      ])
    ).filter(Boolean);
    const bundledLangs = Array.from(
      /* @__PURE__ */ new Set([...options.bundledLangs || [], options.defaultLang])
    ).filter(Boolean);
    const highlightOptions = !options.defaultTheme || typeof options.defaultTheme === "string" ? {
      lang: options.defaultLang || bundledLangs[0] || "javascript",
      theme: options.defaultTheme || bundledThemes[0] || "min-dark",
      ...options.highlightOptions
    } : {
      lang: options.defaultLang || bundledLangs[0] || "javascript",
      themes: {
        ...options.defaultTheme,
        light: options.defaultTheme.light || bundledThemes[0] || "min-light",
        dark: options.defaultTheme.dark || bundledThemes[1] || bundledThemes[0] || "min-dark"
      },
      ...options.highlightOptions
    };
    const template = addTemplate({
      filename: "shiki-config.mjs",
      getContents: () => {
        return `
export async function loadShikiConfig() {
  const [themes, langs] = await Promise.all([
    Promise.all([
${bundledThemes.map((theme) => `${" ".repeat(6)}import("shiki/themes/${theme}.mjs"),`).join("\n")}
    ]),
    Promise.all([
${bundledLangs.map((lang) => `${" ".repeat(6)}import("shiki/langs/${lang}.mjs"),`).join("\n")}
    ]),
  ]);
  return {
    highlightOptions: ${JSON.stringify(highlightOptions)},
    coreOptions: {
      themes,
      langs,
      langAlias: ${JSON.stringify(options.langAlias)},
    },
  };
}`;
      }
    });
    nuxt.options.nitro.virtual = nuxt.options.nitro.virtual || {};
    nuxt.options.nitro.virtual["shiki-config.mjs"] = template.getContents;
    nuxt.options.alias["shiki-config.mjs"] = template.dst;
  }
});
function addUnwasmSupport(nuxt) {
  nuxt.hook("ready", () => {
    const nitro = useNitro();
    const addWasmSupport = (_nitro) => {
      if (nitro.options.experimental?.wasm) {
        return;
      }
      _nitro.options.externals = _nitro.options.externals || {};
      _nitro.options.externals.inline = _nitro.options.externals.inline || [];
      _nitro.options.externals.inline.push((id) => id.endsWith(".wasm"));
      _nitro.hooks.hook("rollup:before", async (_, rollupConfig) => {
        const { rollup: unwasm } = await import('unwasm/plugin');
        rollupConfig.plugins = rollupConfig.plugins || [];
        rollupConfig.plugins.push(
          unwasm({
            ..._nitro.options.wasm
          })
        );
      });
    };
    addWasmSupport(nitro);
    nitro.hooks.hook("prerender:init", (prerenderer) => {
      addWasmSupport(prerenderer);
    });
  });
}

export { module as default };
